#ifdef DO_LP_STUFF
/*
 * LPC storage slots
 */

static  int16	currentLPCSlot=0 ;
#define MAX_LPC_SLOT 20

static	LPREAD	*lprdadr[MAX_LPC_SLOT];
static	char	lpfilname[MAXNAME];


/*
 *
 * LPREAD opcode : initialisation phase
 *
 *
 */

void lprdset(LPREAD *p)
{
        LPHEADER *lph;
	MEMFIL	 *mfp;
	int32     magic;
	int32	 totvals;  /* NB - presumes sizeof(Number) == sizeof(long) !! */
	int32 filno;

	/* Store adress of opcode for other lpXXXX init to point16 to */
	lprdadr[currentLPCSlot] = p;

	/* Build file name */
        if (*p->ifilno == sstrcod) { /* if char string name given */
		extern EVTBLK *currevent;
		extern char *unquote(char *name);
		if (p->STRARG == NULL) strcpy(lpfilname,unquote(currevent->strarg));
		else strcpy(lpfilname, unquote(p->STRARG));
        }
	else if ((filno = (long)*p->ifilno) < strsmax && strsets != NULL &&
		 strsets[filno])
	    strcpy(lpfilname, strsets[filno]);
	else
	    sprintf(lpfilname,"lp.%ld",filno);

	/* Do not reload existing file ? */
        if ((mfp = p->mfp) != NULL && strcmp(mfp->filename,lpfilname) == 0)
	    goto lpend;                             /* rtn if file prv known */
	/* Load analysis in memory file */
	if ((mfp = ldmemfile(lpfilname)) == NULL) {     /* else read file  */
		p->error ("LPREAD cannot load %s",lpfilname);
		goto lperr;
	}
	/* Store memory file location in opcode */
	p->mfp = mfp;                                   /*  & record facts   */
	/* Take a peek to the header if exisiting. Else take input arguments */
	lph = (LPHEADER *) mfp->beginp;

	magic=lph->lpmagic;
	if ((magic==LP_MAGIC)||(magic==LP_MAGIC2)) {
		p->storePoles = (magic==LP_MAGIC2);

		printf("Using %s type of file.\n",p->storePoles?"pole":"filter coefficient");
		/* Store header length */
		p->headlongs = lph->headersize/sizeof(long);
		/* Check if input values where available */
		if (*p->inpoles || *p->ifrmrate)
		    warning << "lpheader overriding inputs" << endmsg;

		/* Check orc/analysis sample rate compatibility */
		if (lph->srate != esr)
		    warning << "lpfile srate != orch sr" << endmsg;

		p->npoles = lph->npoles;		/* note npoles, etc. */
		/* Store header info in opcode */
		p->nvals = lph->nvals;
		p->framrat16 = lph->framrate * 65536.0f;/* scaled framno cvt */
	}
	else if (BYTREVL(lph->lpmagic) == LP_MAGIC) {	/* Header reversed:  */
		p->error ("file %s bytes are in wrong order",lpfilname);
		goto lperr;
	}
	else {						/* No Header on file:*/
		p->headlongs = 0;
		p->npoles = (long)*p->inpoles;		/*  data from inargs */
		p->nvals = p->npoles + 4;
		p->framrat16 = *p->ifrmrate * 65536.0f;
		if (!p->npoles || !p->framrat16) {
			p->error ("insufficient args and no file header");
			goto lperr;
		}
	}
	/* Check  pole number */
	if (p->npoles > MAXPOLES) {
		p->error ("npoles > MAXPOLES");
		goto lperr;
	}
	/* Look for total frame data size (file size - header) */
	totvals = (mfp->length/sizeof(long)) - p->headlongs;        /* see NB above!! */
	/* Store the size of a frame in integer */
	p->lastfram16 = (((totvals - p->nvals) / p->nvals) << 16) - 1;
        VMSG(printf("npoles %ld, nvals %ld, totvals %ld, lastfram16 = %lx\n",
		    p->npoles, p->nvals, totvals, p->lastfram16); )
		lpend:	p->lastmsg = 0;
	return;

}

/*
 *
 * LPREAD k/a time access. This will setup current pole values
 *
 */

extern int16 DoPoleInterpolation(int, Number *, Number *, Number *, Number *, Number, Number *, Number *);

void lpread(LPREAD *p)
{
        Number	*bp, *np, *cp;
	int32	nn, framphase;
	Number	fract;
	int16     i, status;
	Number   poleMagn1[MAXPOLES], polePhas1[MAXPOLES];
	Number   poleMagn2[MAXPOLES], polePhas2[MAXPOLES];
	Number   interMagn[MAXPOLES], interPhas[MAXPOLES];

        if (p->mfp==NULL) {
		initerror("lpread: not initialized");
		return;
        }
	/* Locate frame position range */
	if ((framphase = (long)(*p->ktimpt * p->framrat16)) < 0) { /* for kfram reqd */
		perferror("lpread timpnt < 0");
		return;
	}
	if (framphase > p->lastfram16) {		/* not past last one */
		framphase = p->lastfram16;
		if (!p->lastmsg) {
			p->lastmsg = 1;
			warning << "lpread ktimpnt truncated to last frame" << endmsg;

		}
	}
	/* Locate frames bounding current time */
	nn = (framphase >> 16) * p->nvals + p->headlongs;        /* see NB above!! */
	bp = (Number *)p->mfp->beginp + nn;		/* locate begin this frame */
	np = bp + p->nvals;		                /* & interp betw adj frams */
	fract = (framphase & 0x0FFFFL) / 65536.0f;
	/* Interpolate freq/amplpitude and store in opcode */
	*p->krmr = *bp + (*np - *bp) * fract;	bp++;	np++; /* for 4 rslts */
	*p->krmo = *bp + (*np - *bp) * fract;	bp++;	np++;
	*p->kerr = *bp + (*np - *bp) * fract;	bp++;	np++;
	*p->kcps = *bp + (*np - *bp) * fract;	bp++;	np++;

	/* Interpolate filter or poles coef values and store in opcode */

	cp = p->kcoefs;      /* This is where the coefs get stored */
	if (p->storePoles) {
		for (i=0; i<p->npoles; i++) {
			poleMagn1[i] = *bp++;
			polePhas1[i] = *bp++;
			poleMagn2[i] = *np++;
			polePhas2[i] = *np++;
		}

		status =
			DoPoleInterpolation(p->npoles,poleMagn1,polePhas1,poleMagn2,
					    polePhas2,fract,interMagn,interPhas);
		if (!status) perferror("Interpolation failed\n");

		for (i=0; i<p->npoles; i++) {
			*cp++ = interMagn[i];
			*cp++ = interPhas[i];
		}
	}
	else {
		nn = p->npoles;
		do {
			*cp = *bp + (*np - *bp) * fract;
			cp++; bp++; np++;
		}
		while (--nn);
	}
/*      VMSG ({
	printf("phase:%lx fract:%6.2f rmsr:%6.2f rmso:%6.2f kerr:%6.2f kcps:%6.2f\n",
	framphase,fract,*p->krmr,*p->krmo,*p->kerr,*p->kcps);
	cp = p->kcoefs;
	nn = p->npoles;
	do    printf(" %6.2f",*cp++);
	while (--nn);
	putchar('\n'); if (dribble) putc('\n', dribble);
	})  */
}

/*
 *
 * LPRESON: initialisation time
 *
 *
 */
void lprsnset(LPRESON *p)
{
        LPREAD *q;

	/* connect to previously loaded lpc analysis */
	p->lpread = q = lprdadr[currentLPCSlot];		    /* get adr lpread struct */

	/* Initialize pointer to circulat buffer (for filtering) */
	p->circjp = p->circbuf;
	p->jp2lim = p->circbuf + (q->npoles << 1);  /* npoles det circbuflim */
}

/*
 *
 * LPRESON: k & a time access. Will actually filter the signal
 *		    Uses a circular buffer to store previous signal values.
 */

void DumpPoles(int, double *, double *, int, char *);
void InvertPoles(int, double *, double *);
void synthetize(int, double *, double *, double *, double *);

void lpreson(LPRESON *p)
{
        LPREAD *q = p->lpread;
        int16	nn, nsmps = ksmps;
	Number	*coefp, *pastp, *jp, *jp2, *rslt = p->ar, *asig = p->asig;
	Number	x;
	double  poleReal[MAXPOLES], poleImag[MAXPOLES];
	double  polyReal[MAXPOLES+1], polyImag[MAXPOLES+1];
	int16     i;
	double  pm,pp;

	jp = p->circjp;
	jp2 = jp + q->npoles;

	/* If we where using poles, we have to compute filter coefs now */
	if (q->storePoles) {
		coefp = q->kcoefs;
		for (i=0; i<q->npoles; i++) {
			pm = *coefp++;
			pp = *coefp++;
			if (fabs(pm)>0.999999)
			    pm = 1/pm;
			poleReal[i] = pm*cos(pp);
			poleImag[i] = pm*sin(pp);

		}
/* 		DumpPoles(q->npoles,poleReal,poleImag,0,"About to filter"); */
		InvertPoles(q->npoles,poleReal,poleImag);
		synthetize(q->npoles,poleReal,poleImag,polyReal,polyImag);
		coefp = q->kcoefs;
		for (i=0; i<q->npoles; i++) {
			coefp[i] = -(Number)polyReal[q->npoles-i]; /* MR_WHY - somthing with the atan2 ? */
#ifdef _DEBUG
			/*			if (polyImag[i]>1e-10) */
			/*			{ */
			/*				printf ("bad polymag: %f\n",polyImag[i]); */
			/*			} */
#endif
		}
	}

	/* For each sample */
	do {
		/* Compute Xn = Yn + CkXn-k */

#ifdef TRACE_FILTER
		printf ("Asig=%f\n",*asig);
#endif
		x = *asig++;
		coefp = q->kcoefs;		/* using lpread interp coefs */
		pastp = jp;
		nn = q->npoles;
		do { 
#ifdef TRACE_FILTER
			printf ("\t%f,%f\n",*coefp,*pastp);
#endif
			x += *coefp++ * *pastp++;
		} while (--nn);
#ifdef TRACE_FILTER
		printf ("result=%f\n",x);
#endif
		/* Store result signal in circular and output buffers */

		*jp++ = *jp2++ = x;
		*rslt++ = x;

		/* Check if end of buffer reached */
		if (jp2 >= p->jp2lim) {
			jp2 = jp;
			jp = p->circbuf;
		}
	} while (--nsmps);
	p->circjp = jp;
}

/*
 *
 * LPFRESON : Initialisation time
 *
 */
void lpfrsnset(LPFRESON *p)
{

	/* Connect to previously loaded analysis file */

	if (lprdadr[currentLPCSlot]->storePoles)
	    initerror("Pole file not supported for this opcode !\n");
	p->lpread = lprdadr[currentLPCSlot];
	p->prvratio = 1.0f;
	p->d = 0.0f;
	p->prvout = 0.0f;
}

/*
 *
 * LPFRESON : k & a time : actually filters the data
 *
 */
void lpfreson(LPFRESON *p)
{
        LPREAD	*q = p->lpread;
	int16	nn, nsmps = ksmps;
	Number	*coefp, *pastp, *pastp1, *rslt = p->ar, *asig = p->asig;
	Number	x, temp1, temp2, ampscale, cq;

	if (*p->kfrqratio != p->prvratio) {		/* for new freqratio */
		if (*p->kfrqratio <= 0.) {
			p->error ("illegal frqratio, %5.2f",*p->kfrqratio);
			return;
		}					/*	calculate d  */
		p->d = (*p->kfrqratio - 1.0f) / (*p->kfrqratio + 1.0f);
		p->prvratio = *p->kfrqratio;
	}
	if (p->d != 0.) {				/* for non-zero d,   */
		coefp = q->kcoefs;
		nn = q->npoles - 1;
		do {
			temp1 = p->d * *coefp++;	/*    shift formants */
			*coefp += temp1;
		}
		while (--nn);
		ampscale = 1.0f / (1.0f - p->d * *coefp);	/*    & reset scales */
		cq = (1.0f - p->d * p->d) * ampscale;
	}
	else {
		cq = 1.0f;
		ampscale = 1.0f;
	}
	x = p->prvout;
	do {
		nn = q->npoles - 1;
		pastp  = pastp1 = p->past + nn;
		temp1 = *pastp;
		*pastp = cq * x - p->d * *pastp;
		pastp--;
		do {
			temp2 = *pastp;
			*pastp = (*pastp1 - *pastp) * p->d + temp1;
			pastp--;   pastp1--;
			temp1 = temp2;
		}
		while (--nn);
		x = *asig++;
		pastp = p->past;
		coefp = q->kcoefs;
		nn = q->npoles;
		do  x += *coefp++ * *pastp++;
		while (--nn);
		*rslt++ = x * ampscale;
	}
	while (--nsmps);
	p->prvout = x;
}


#if 0			/* This is incorrect C */
	if (&p->kcoefs-p != &p->lp1->kcoefs-p->lp1)
	    initerror("padding error");
#endif

	p->npoles = p->lp1->npoles;
	p->storePoles = 1;
	lprdadr[currentLPCSlot] = (LPREAD*)p;
}

void lpinterpol(LPINTERPOL *p)
{
        int16	i,status;
	Number	*cp,*cp1,*cp2;
	Number	poleMagn1[MAXPOLES], polePhas1[MAXPOLES];
	Number	poleMagn2[MAXPOLES], polePhas2[MAXPOLES];
	Number	interMagn[MAXPOLES], interPhas[MAXPOLES];

        /* RWD: guessing this... */
        if (p->lp1==NULL || p->lp2==NULL) {
		initerror("lpinterpol: not initialized");
		return;
        }
	cp1 =  p->lp1->kcoefs;
	cp2 =  p->lp2->kcoefs;

	for (i=0; i<p->npoles; i++) {
		poleMagn1[i] = *cp1++;
		polePhas1[i] = *cp1++;
		poleMagn2[i] = *cp2++;
		polePhas2[i] = *cp2++;
	}

	status = DoPoleInterpolation(p->npoles,poleMagn1,polePhas1,poleMagn2,
				     polePhas2,*p->kmix,interMagn,interPhas);
	if (!status) perferror("Interpolation failed\n");

	cp = p->kcoefs;      /* This is where the coefs get stored */
	for (i=0; i<p->npoles; i++) {
		*cp++ = interMagn[i];
		*cp++ = interPhas[i];
	}
}


/*
 *   Set current lpc slot
 */
void lpslotset(LPSLOT *p)
{
        int16 n;

	n = (int)*(p->islotnum);
	if (n<0) 
	    initerror("lpslot number should be positive\n");
	if (n>MAX_LPC_SLOT) 
	    initerror("lpslot number should be less than 20\n");
	currentLPCSlot = n;
}

void lpitpset(LPINTERPOL *p)
{

	/* Get lpread pointers */
	p->lp1 = lprdadr[(int)*(p->islot1)];
	p->lp2 = lprdadr[(int)*(p->islot2)];

	/* Check if workable */

	if ((!p->lp1->storePoles) || (!p->lp2->storePoles))
	    initerror("lpinterpol works only with poles files..");

	if (p->lp1->npoles != p->lp2->npoles)
	    initerror("The poles files have different pole count\n");
}


#endif DO_LP_STUFF
